#include "gfxinterface.h"
#include <cmath>
#include <stdexcept>


/*
 *  Coord
 */


Coord::Coord(RCoord rc)
{
	coord_.x = UnRel(rc.X(), X11::Inst().WindowWidth());
	coord_.y = UnRel(rc.Y(), X11::Inst().WindowHeight());
}

RCoord Coord::ToRCoord() const
{
	return RCoord(Rel(coord_.x, X11::Inst().WindowWidth()),
	              Rel(coord_.y, X11::Inst().WindowHeight()));
}

bool Coord::XOut() const
{
	return coord_.x < 0 || coord_.x >= X11::Inst().WindowWidth();
}

bool Coord::YOut() const
{
	return coord_.y < 0 || coord_.y >= X11::Inst().WindowHeight();
}

double Coord::Length() const
{
	return std::sqrt(static_cast<double>(coord_.x * coord_.x +
	                                     coord_.y * coord_.y));
}



/*
 *  X11
 */

X11 *    X11::singleton_             (0);
unsigned X11::default_window_width_  (400);
unsigned X11::default_window_height_ (512);


X11::X11()
: window_width_ (default_window_width_)
, window_height_(default_window_height_)
{
	display_ = XOpenDisplay(0);
	if (!display_) throw std::runtime_error("Cannot open display");
	screen_   = DefaultScreen(display_);
	colormap_ = DefaultColormap(display_, screen_);
	gc_       = DefaultGC(display_, screen_);

	const Color black (BlackPixel(display_, screen_));
	window_ = XCreateSimpleWindow(display_, DefaultRootWindow(display_),
	                              0, 0, window_width_, window_height_,
				      0, black, black);
	XSelectInput(display_, window_, StructureNotifyMask);
	XMapWindow(display_, window_);
	XEvent e;
	do XNextEvent(display_, &e); while (e.type != MapNotify);
	XSelectInput(display_, window_, KeyPressMask | KeyReleaseMask);
	XWindowAttributes attrs;
	XGetWindowAttributes(display_, window_, &attrs);
	window_width_  = attrs.width;
	window_height_ = attrs.height;
}


X11::~X11()
{
	XDestroyWindow(display_, window_);
	XCloseDisplay(display_);
}


void X11::SetWindowTitle(const std::string & title) const
{
	const char *const list_ptr (title.c_str());
 	XTextProperty property;
	if (XStringListToTextProperty(const_cast<char**>(&list_ptr), 1, &property)) {
		XSetWMName(display_, window_, &property);
		XFree(property.value);
	} // Do nothing if out of memory.
}


Status X11::GetWindowAttributes(XWindowAttributes * attrs)
{
	XWindowAttributes attrs2;
	if (!attrs) attrs = &attrs2;
	const Status s (XGetWindowAttributes(display_, window_, attrs));
	if (s != 0) {
		window_width_  = attrs->width;
		window_height_ = attrs->height;
	}
	return s;
}
